import pytest
import os
from b2constsites.b2constsites import B2ConstSites


test_folder = os.path.dirname(os.path.realpath(__file__))
test_seq = os.path.join(test_folder, 'test.fa')
test_vcf = os.path.join(test_folder, 'test.vcf')
test_bed = os.path.join(test_folder, 'test.bed')

test_seq_withN = os.path.join(test_folder, 'test_withN.fa')
test_seq_withDash = os.path.join(test_folder, 'test_withdash.fa')

b2cs_bed = B2ConstSites(seq=test_seq,
                        vcf=test_vcf,
                        maskbed=test_bed,
                        fmt='fasta')


b2cs_nobed = B2ConstSites(seq=test_seq,
                          vcf=test_vcf,
                          fmt='fasta')

b2cs_withN = B2ConstSites(seq=test_seq_withN,
                          vcf=test_vcf,
                          fmt='fasta')

b2cs_withDash = B2ConstSites(seq=test_seq_withDash,
                             vcf=test_vcf,
                             fmt='fasta')


def test_load_seq_with_bed():
    b2cs_bed.load_seq()
    assert len(b2cs_bed.seqrec) == 16


def test_load_vcf_with_bed():
    b2cs_bed.load_vcf()
    assert {2, 6, 10, 11} == b2cs_bed.var_pos


def test_load_bed_with_bed():
    b2cs_bed.load_mask()
    assert [range(4, 7)] == b2cs_bed.mask_pos


def test_const_sites_A_with_bed():
    b2cs_bed.const_sites()
    assert 2 == b2cs_bed.tally['A']


def test_const_sites_T_with_bed():
    b2cs_bed.const_sites()
    assert 2 == b2cs_bed.tally['T']


def test_const_sites_C_with_bed():
    b2cs_bed.const_sites()
    assert 2 == b2cs_bed.tally['C']


def test_const_sites_G_with_bed():
    b2cs_bed.const_sites()
    assert 4 == b2cs_bed.tally['G']


def test_output_with_bed(capfd):
    print(b2cs_bed)
    out, err = capfd.readouterr()
    assert out == "<data id='xyz' spec='FilteredAlignment' filter='-'"\
                  " data='@xyzOriginal'"\
                  " constantSiteWeights='2 2"\
                  " 4 2'/>\n"


def test_load_seq_without_bed():
    b2cs_nobed.load_seq()
    assert len(b2cs_nobed.seqrec) == 16


def test_load_vcf_without_bed():
    b2cs_nobed.load_vcf()
    assert {2, 6, 10, 11} == b2cs_nobed.var_pos


def test_load_bed_without_bed():
    b2cs_nobed.load_mask()
    assert b2cs_nobed.mask_pos is None


def test_const_sites_A_without_bed():
    b2cs_nobed.const_sites()
    assert 3 == b2cs_nobed.tally['A']


def test_const_sites_T_without_bed():
    b2cs_nobed.const_sites()
    assert 3 == b2cs_nobed.tally['T']


def test_const_sites_C_without_bed():
    b2cs_nobed.const_sites()
    assert 2 == b2cs_nobed.tally['C']


def test_const_sites_G_without_bed():
    b2cs_nobed.const_sites()
    assert 4 == b2cs_nobed.tally['G']


def test_output_with_nobed(capfd):
    print(b2cs_nobed)
    out, err = capfd.readouterr()
    assert out == "<data id='xyz' spec='FilteredAlignment' filter='-'"\
                  " data='@xyzOriginal'"\
                  " constantSiteWeights='3 2"\
                  " 4 3'/>\n"


def test_class_seq_withN(capfd):
    b2cs_withN.load_seq()
    b2cs_withN.load_vcf()
    b2cs_withN.load_mask()
    b2cs_withN.const_sites()
    print(b2cs_withN)
    out, err = capfd.readouterr()
    assert out == "<data id='xyz' spec='FilteredAlignment' filter='-'"\
                  " data='@xyzOriginal'"\
                  " constantSiteWeights='3 1"\
                  " 4 3'/>\n"


def test_class_seq_withDash(capfd):
    b2cs_withDash.load_seq()
    b2cs_withDash.load_vcf()
    b2cs_withDash.load_mask()
    b2cs_withDash.const_sites()
    print(b2cs_withDash)
    out, err = capfd.readouterr()
    assert out == "<data id='xyz' spec='FilteredAlignment' filter='-'"\
                  " data='@xyzOriginal'"\
                  " constantSiteWeights='3 2"\
                  " 4 3'/>\n"



import click
from .b2constsites import B2ConstSites


@click.command()
@click.argument('seq')
@click.argument('vcf')
@click.option('-m', '--maskbed', default=None,
              help='A BED file with positions to mask')
@click.option('-f', '--fmt', default='fasta',
              help='seq sequence format', show_default=True)
def run_b2constsites(seq, vcf, maskbed, fmt):
    b2cs = B2ConstSites(seq=seq,
                        vcf=vcf,
                        maskbed=maskbed,
                        fmt=fmt)
    b2cs.load_seq()
    b2cs.load_vcf()
    b2cs.load_mask()
    b2cs.const_sites()
    print(b2cs)
    pass


if __name__ == '__main__':
    run_b2constsites()



import vcf
from Bio import SeqIO
import os
import logging
import collections


class B2ConstSites:
    def __init__(self,
                 seq,
                 vcf,
                 maskbed=None,
                 fmt='fasta'):
        self.seq = os.path.realpath(seq)
        self.seq_format = fmt
        self.vcf = os.path.realpath(vcf)
        if maskbed is not None:
            self.mask = os.path.realpath(maskbed)
        else:
            self.mask = None

    def load_seq(self):
        fh = open(self.seq, 'rt')
        self.seqrec = SeqIO.read(fh, format=self.seq_format)
        logging.info(f'Imported sequence {self.seqrec.id}.')
        logging.info(f'it has length {len(self.seqrec)}')
        fh.close()
        pass

    def _parse_pos(self, rec):
        '''
        Find which positions to mask
        '''
        ACCEPTABLE_VAR_TYPES = {'snp', 'mnp'}
        try:
            if rec.INFO['TYPE'][0] in ACCEPTABLE_VAR_TYPES:
                start_pos = rec.POS
                if rec.INFO['TYPE'][0] == 'mnp':
                    end_pos = start_pos + len(rec.ALT[0])
                else:
                    end_pos = start_pos + 1
        except KeyError:
            logging.critical("Something is not right with your vcf file")
            raise
        return list(range(start_pos, end_pos))

    def load_vcf(self):
        fh = open(self.vcf, 'rt')
        self.var_pos = set()
        for rec in vcf.Reader(fh):
            self.var_pos.update(self._parse_pos(rec))
        fh.close()
        logging.info(f'Loaded {len(self.var_pos)} SNPs from vcf.')
        pass

    def load_mask(self):
        if self.mask is not None:
            fh = open(self.mask, 'rt')
            mask_pos = [line.strip().split('\t') for line in fh]
            self.mask_pos = [range(int(start)+1, int(end))
                             for chrom, start, end in mask_pos]
        else:
            self.mask_pos = None
        pass

    def const_sites(self):
        tally = collections.Counter({'A': 0,
                                     'C': 0,
                                     'G': 0,
                                     'T': 0})
        for p, b in enumerate(self.seqrec):
            p += 1
            if self.mask_pos is not None\
                    and any([p in r for r in self.mask_pos])\
                    or p in self.var_pos:
                continue
            else:
                tally[b] += 1
        self.tally = tally
        for b in self.tally:
            logging.info(f"Found {self.tally[b]} occurrences of {b}!")
        pass

    def __str__(self):
        return f"<data id='xyz' spec='FilteredAlignment' filter='-'"\
               f" data='@xyzOriginal'"\
               f" constantSiteWeights='{self.tally['A']} {self.tally['C']}"\
               f" {self.tally['G']} {self.tally['T']}'/>"



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['b2constsites']

package_data = \
{'': ['*']}

install_requires = \
['biopython>=1.70', 'pyvcf>=0.6.8']

entry_points = \
{'console_scripts': ['run_b2cs = '
                     'b2constsites.run_b2constsites:run_b2constsites']}

setup(name='b2constsites',
      version='0.1',
      description='Generate an appropriate data tag to add constant sites to your BEAST2 XML',
      author='Anders Goncalves da Silva',
      author_email='andersgs@gmail.com',
      url='https://github.com/andersgs/beast2_constsites/',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
